@@ -7,6 +7,7 @@
 #include <string>
 #include <vector>
 #include <filesystem>
+#include <set>
 
 #include <pl/api.hpp>
 
@@ -22,10 +23,6 @@
 #include <wolv/io/fs.hpp>
 #include <wolv/container/interval_tree.hpp>
 
-namespace hex::prv {
-    class Provider;
-}
-
 namespace pl {
 
     namespace core {
@@ -95,7 +92,7 @@ namespace pl {
          * @param checkResult Whether to check the result of the execution
          * @return True if the execution was successful, false otherwise. Call PatternLanguage#getCompileErrors() AND PatternLanguage#getEvalError() to get the compilation or runtime errors if false is returned
          */
-        [[nodiscard]] bool executeString(std::string code, const std::string& source, const std::map<std::string, core::Token::Literal> &envVars = {}, const std::map<std::string, core::Token::Literal> &inVariables = {}, bool checkResult = true);
+        [[nodiscard]] bool executeString(std::string code, const std::string& source = api::Source::DefaultSource, const std::map<std::string, core::Token::Literal> &envVars = {}, const std::map<std::string, core::Token::Literal> &inVariables = {}, bool checkResult = true);
 
         /**
          * @brief Executes a pattern language file
@@ -120,7 +117,7 @@ namespace pl {
          * @param source the source of the code
          * @return the source that was added or that already existed
          */
-        [[nodiscard]] api::Source* addVirtualSource(const std::string& code, const std::string& source) const;
+        [[nodiscard]] api::Source* addVirtualSource(const std::string& code, const std::string& source, bool mainSource = false) const;
 
         /**
          * @brief Aborts the currently running execution asynchronously
@@ -134,19 +131,19 @@ namespace pl {
          * @param readFunction Function to read data from the data source
          * @param writerFunction Optional function to write data to the data source
          */
-        void setDataSource(u64 baseAddress, u64 size, std::function<void(u64, u8*, size_t)> readFunction, std::optional<std::function<void(u64, const u8*, size_t)>> writerFunction = std::nullopt) const;
+        void setDataSource(u64 baseAddress, u64 size, std::function<void(u64, u8*, size_t)> readFunction, std::optional<std::function<void(u64, const u8*, size_t)>> writerFunction = std::nullopt);
 
         /**
          * @brief Sets the base address of the data source
          * @param baseAddress Base address of the data source
          */
-        void setDataBaseAddress(u64 baseAddress) const;
+        void setDataBaseAddress(u64 baseAddress);
 
         /**
          * @brief Sets the size of the data source
          * @param size Size of the data source
          */
-        void setDataSize(u64 size) const;
+        void setDataSize(u64 size);
 
         /**
          * @brief Sets the default endianess used in the pattern language
@@ -159,6 +156,7 @@ namespace pl {
          * @param address Initial cursor position
          */
         void setStartAddress(u64 address);
+        u64 getStartAddress() const;
 
 
         /**
@@ -191,7 +189,7 @@ namespace pl {
          * @brief Sets the include paths for where to look for include files
          * @param paths List of paths to look in
          */
-        void setIncludePaths(const std::vector<std::fs::path>& paths) const;
+        void setIncludePaths(const std::vector<std::fs::path>& paths);
 
         /**
          * @brief Sets the source resolver of the pattern language
@@ -206,13 +204,13 @@ namespace pl {
          * @note If the callback is not set, dangerous functions are disabled
          * @param callback Callback to call
          */
-        void setDangerousFunctionCallHandler(std::function<bool()> callback) const;
+        void setDangerousFunctionCallHandler(std::function<bool()> callback);
 
         /**
          * @brief Sets the console log callback
          * @param callback Callback to call
          */
-        void setLogCallback(const core::LogConsole::Callback &callback) const;
+        void setLogCallback(const core::LogConsole::Callback &callback);
 
         /**
          * @brief Gets the potential error that occurred during the last evaluation of the compiled AST. This does NOT include compilation errors.
@@ -309,7 +307,7 @@ namespace pl {
          * @param parameterCount Number of parameters the function takes
          * @param func Callback to execute when the function is called
          */
-        void addFunction(const api::Namespace &ns, const std::string &name, api::FunctionParameterCount parameterCount, const api::FunctionCallback &func) const;
+        void addFunction(const api::Namespace &ns, const std::string &name, api::FunctionParameterCount parameterCount, const api::FunctionCallback &func);
 
         /**
          * @brief Adds a new dangerous built-in function to the pattern language
@@ -318,7 +316,16 @@ namespace pl {
          * @param parameterCount Number of parameters the function takes
          * @param func Callback to execute when the function is called
          */
-        void addDangerousFunction(const api::Namespace &ns, const std::string &name, api::FunctionParameterCount parameterCount, const api::FunctionCallback &func) const;
+        void addDangerousFunction(const api::Namespace &ns, const std::string &name, api::FunctionParameterCount parameterCount, const api::FunctionCallback &func);
+
+        /**
+         * @brief Adds a new custom built-in type to the pattern language
+         * @param ns Namespace of the type
+         * @param name Name of the type
+         * @param parameterCount Number of non-type template parameters the type takes
+         * @param func Callback to execute when the type is instantiated
+         */
+        void addType(const api::Namespace &ns, const std::string &name, api::FunctionParameterCount parameterCount, const api::TypeCallback &func);
 
         /**
          * @brief Gets the internals of the pattern language
@@ -386,16 +393,24 @@ namespace pl {
         [[nodiscard]] const std::atomic<u64>& getLastWriteAddress() const;
         [[nodiscard]] const std::atomic<u64>& getLastPatternPlaceAddress() const;
 
+        PatternLanguage cloneRuntime() const;
+
+        [[nodiscard]] bool isSubRuntime() const {
+            return this->m_subRuntime;
+        }
+
+        [[nodiscard]] const std::set<pl::ptrn::Pattern*>& getPatternsWithAttribute(const std::string &attribute) const;
+
     private:
         void flattenPatterns();
 
     private:
-
         Internals m_internals;
         std::vector<core::err::CompileError> m_compileErrors;
         std::optional<core::err::PatternLanguageError> m_currError;
         std::map<std::string, std::string> m_defines;
         std::map<std::string, api::PragmaHandler> m_pragmas;
+        bool m_subRuntime = false;
 
         core::Resolver m_resolvers;
         core::resolvers::FileResolver m_fileResolver;
@@ -414,6 +429,23 @@ namespace pl {
         std::optional<u64> m_startAddress;
         std::endian m_defaultEndian = std::endian::little;
         double m_runningTime = 0;
+
+        u64 m_dataBaseAddress;
+        u64 m_dataSize;
+        std::function<void(u64, u8*, size_t)> m_dataReadFunction;
+        std::optional<std::function<void(u64, const u8*, size_t)>> m_dataWriteFunction;
+
+        std::function<bool()> m_dangerousFunctionCallCallback;
+        core::LogConsole::Callback m_logCallback;
+
+        struct Function {
+            api::Namespace nameSpace;
+            std::string name;
+            api::FunctionParameterCount parameterCount;
+            api::FunctionCallback callback;
+            bool dangerous;
+        };
+        std::vector<Function> m_functions;
     };
 
 }
\ No newline at end of file
